/* Aine knowledge files compiler

 License: GPL
 This is part of the Aine Project.
 For copyrights and license (GPL) - see main.c in main source dir
 
*/

#include <stdarg.h>
#include "aine_c.h"
#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif

#undef FALSE
#undef TRUE
typedef enum { FALSE, TRUE } bool;

FILE *indexpos_f, *inputs_a_f, *inputs_a_t, *inputs_d_f, *inputs_d_t, *aine;

char *topic;   /* Topic holds our topic which we convert to conditional tags */
FILE *index_f, *topicfile, *unsorted_a_f, *unsorted_a_t, *unsorted_d_f;
FILE *unsorted_d_t, *outputs_f;
unsigned long size; /* number of categories bot have */


static void print_usage (char name[]) {
	puts("************************************");
	puts("       AineL file compiler");
	puts("************************************\n");
	puts("Usage:");
	printf("%s brain_index_file\n", name);
}


/* prints out panic reason,
 * deletes all created files
 * (as they are most probably broken, if compiler encountered a problem)
 * and quits
 */
void panic (char *s, ...) {
	va_list argp;
	va_start (argp, s);
	vfprintf (stderr, s, argp);
	va_end (argp);
	fprintf (stderr, ".\n");

	remove ("data/brain/inputs_a_f");
	remove ("data/brain/inputs_a_t");
	remove ("data/brain/inputs_d_f");
	remove ("data/brain/inputs_d_t");
	remove ("data/brain/outputs");
	remove ("data/brain/topics");
	remove ("data/brain/indexpos");
	remove ("data/brain/unsorted_a_t");
	remove ("data/brain/unsorted_a_f");	
	remove ("data/brain/unsorted_d_t");
	remove ("data/brain/unsorted_d_f");
	exit (1);
}


void wardens_writer (FILE *out) {
	int i, j;
	char bla[] = "blabla<$=0";
	for (j = 0; j < 4; j++) {
		i = -1;
		fwrite (&i, sizeof(int), 1, out);
		i = strlen (bla) +1;
		fwrite (&i, sizeof(int), 1, out);
		i = 6;
		fwrite (&i, sizeof(int), 1, out);
		fwrite (bla, sizeof(char), i, out);
	}
}


int main (int argc, char *argv[])
{
#ifdef HAVE_LOCALE_H
	setlocale (LC_ALL, "");
#endif
	
	if (argc > 1) {
		index_f = fopen (argv[1], "r");
		if (!index_f) {
			printf ("I cannot open %s file\n", argv[1]);
			print_usage(argv[0]);
			exit (1);
		}
	} else {
		print_usage(argv[0]);
		exit (0);
    }
	
	puts ("I must have permission to write to /data, where I will drop compiled files for Aine");
	
	inputs_a_f = fopen ("data/brain/inputs_a_f", "w");
	if (!inputs_a_f) {
		fputs ("Cannot create data/brain/inputs_a_f", stderr);
		exit (2);
	}
	inputs_a_t = fopen ("data/brain/inputs_a_t", "w");
	inputs_d_f = fopen ("data/brain/inputs_d_f", "wb");
	inputs_d_t = fopen ("data/brain/inputs_d_t", "w");

	setvbuf (inputs_a_t , NULL , _IOFBF , 16384 );
	setvbuf (inputs_a_f , NULL , _IOFBF , 16384);  
	setvbuf (inputs_d_t , NULL , _IOFBF , 16384 );
	setvbuf (inputs_d_f , NULL , _IOFBF , 16384 );
	
	outputs_f  = fopen ("data/brain/outputs",    "w"); 
	indexpos_f = fopen ("data/brain/indexpos",   "wb");
	topicfile  = fopen ("data/brain/topics",     "wb");
  
	unsorted_a_t = fopen ("data/brain/unsorted_a_t",  "w");
	unsorted_a_f = fopen ("data/brain/unsorted_a_f",  "w");	
	unsorted_d_t = fopen ("data/brain/unsorted_d_t",  "w");
	unsorted_d_f = fopen ("data/brain/unsorted_d_f",  "w");

	size = 0;
	if (prepare()) puts ("OK");
	else puts ("Error");

	printf ("Total patterns = %lu\n", size);
	
	fclose (inputs_a_f);
	fclose (inputs_a_t);
	wardens_writer (inputs_d_f);
	fclose (inputs_d_f);
	fclose (inputs_d_t);	
	
	inputs_a_f = fopen ("data/brain/inputs_a_f", "r");
	inputs_a_t = fopen ("data/brain/inputs_a_t", "r");
	inputs_d_f = fopen ("data/brain/inputs_d_f", "rb");
	inputs_d_t = fopen ("data/brain/inputs_d_t", "r");
	
	puts ("Checking for duplicate patterns...");
	puts ("NOTE: you may have some \"false\" alerts about");
	puts ("\tduplicate patterns that, in fact, has different topics.");
	puts ("\tWhy? Aine cares for topic only in categories without \"that\"");
	puts ("\tbut with a wildcard. (It's most sensible approach, making Aine pretty fast,");
	puts ("\tand making \"topic\" feature works, at the same time,");
	puts ("\ton categories where \"topic\" feature is most useful).");
	
	duplicate_checker (inputs_a_f, 0);
	duplicate_checker (inputs_a_t, 0);
	duplicate_checker (inputs_d_f, 1);
	duplicate_checker (inputs_d_t, 0);

	puts ("OK");
	fclose (index_f);

	fclose (inputs_a_f);
	fclose (inputs_a_t);
	fclose (inputs_d_f);
	fclose (inputs_d_t);
  
	fclose (outputs_f);
	fclose (indexpos_f);
	fclose (topicfile);
  
	fclose (unsorted_a_t);
	fclose (unsorted_a_f);
	fclose (unsorted_d_f);
	fclose (unsorted_d_t);
	
	remove ("data/brain/unsorted_a_t");
	remove ("data/brain/unsorted_a_f");	
	remove ("data/brain/unsorted_d_t");
	remove ("data/brain/unsorted_d_f");
	save_dictionary();

	return (0);
}

